package com.tidecloud.dataacceptance.rfid;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

import com.tidecloud.dataacceptance.rfid.module.interaction.ModuleConnector;
import com.tidecloud.dataacceptance.rfid.module.interaction.ReaderHelper;

/**
 * The implementation class of ModuleConnector.
 *
 */
public class ReaderConnector implements ModuleConnector{
	private  final String HOSTNAME_REGEXP = "^(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|[1-9])\\."
			+ "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
			+ "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
			+ "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)$";
	
	private RFIDReaderHelper mRFIDReaderHelper;
	private Socket mSocket;
	private InetSocketAddress mRemoteAddr;
	
	private final String USER = "USER";
	
	

	@Override
	public ReaderHelper connectNet(final String host, final int port) {

		if (host.matches(HOSTNAME_REGEXP))
			;
		else {
			return null;
		}
		
		try {
			mRemoteAddr = new InetSocketAddress(host, port);
			mSocket = new Socket();
		} catch (Exception e1) {
			return null;
		}

		try {
			mSocket.connect(mRemoteAddr, 4000);
			return connect(mSocket.getInputStream(),mSocket.getOutputStream());
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	@Override
	public ReaderHelper connect(InputStream in,OutputStream out) {
		mRFIDReaderHelper = new RFIDReaderHelper();
		try {
			mRFIDReaderHelper.setReader(in,out,new ReaderDataPackageParser(),new ReaderDataPackageProcess());
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}		
		return mRFIDReaderHelper;
	}

	@Override
	public boolean isConnected() {
		return (mRFIDReaderHelper != null ? mRFIDReaderHelper.isAlive() : false);
	}

	@Override
	public void disConnect() {
		if (mRFIDReaderHelper != null) {
			mRFIDReaderHelper.signOut();
			mRFIDReaderHelper = null;
		}
		try {
			if (mSocket != null) {
				mSocket.close();
				mSocket = null;
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		mRemoteAddr = null;
	}

}
