package com.tidecloud.dataacceptance.service;

import java.io.IOException;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import com.smartsanitation.common.util.StringUtil;
import com.tidecloud.dataacceptance.config.AcceptanceDeviceConfig;
import com.tidecloud.dataacceptance.config.DeviceConfig;
import com.tidecloud.dataacceptance.entity.ConnectMsg;
import com.tidecloud.dataacceptance.service.impl.BingShuiGpsServerHandler;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.timeout.IdleStateHandler;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

/**
 * Created by ryan
 */
@Service
public class AcceptanceDeviceService implements ApplicationContextAware {
	private static Logger logger = LoggerFactory.getLogger(AcceptanceDeviceService.class);
	private ApplicationContext context;

	@Autowired
	private AcceptanceDeviceConfig acceptanceDeviceListConfig;

	@Autowired
	private JedisPool jedisPool;

	@PostConstruct
	public void acceptance() throws IOException, ParseException {

		List<DeviceConfig> deviceListConfig = acceptanceDeviceListConfig.getDeviceList();
		List<DeviceConfig> workDeviceList = new ArrayList<>();
		for (DeviceConfig config : deviceListConfig) {
			if (config.isEnable()) {
				workDeviceList.add(config);
			}
		}

		if (workDeviceList.size() > 0) {
			ExecutorService singleThreadPool = new ThreadPoolExecutor(workDeviceList.size(), 20, 1L,
					TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(1024), Executors.defaultThreadFactory(),
					new java.util.concurrent.ThreadPoolExecutor.AbortPolicy());

			for (int i = 0; i < workDeviceList.size(); i++) {
				DeviceConfig config = workDeviceList.get(i);
				singleThreadPool.execute(() -> {
					
					try {
						Class handlerClass = Class.forName(config.getHandlerClass());
						AcceptanceInboundHandlerAdapter handler = (AcceptanceInboundHandlerAdapter) context
								.getBean(handlerClass);
						handler.setDataPath(config.getDataFileDir());
						handler.setTopic(config.getTopic());
						String ip = getIp("eth0");
						handler.setIp(ip);
						handler.setPort(config.getPort());
						handler.setPrefixName(config.getName());
						handler.startAcceptor();
					} catch (ClassNotFoundException e) {
						logger.error(e.getMessage());
					} catch (Exception ex) {
						logger.error(ex.getMessage());
					}

				});
			}
		}

	}

	

	

	private void cleanRedisLinkData(String address) {

		try (Jedis jedis = jedisPool.getResource()) {
			jedis.select(BingShuiGpsServerHandler.REDIS_INDEX_LINK);
			String addressStr = ConnectMsg.ipToLong(address);
			String selectKey = BingShuiGpsServerHandler.PREFIX_LINK_BACK + addressStr;
			Set<String> values = jedis.smembers(selectKey);

			for (String deviceId : values) {
				String deleteKeyOfDevice = BingShuiGpsServerHandler.PREFIX_DEVICE + deviceId;
				String deleteKeyOfLink = BingShuiGpsServerHandler.PREFIX_LINK + addressStr;
				String connectMsgStr = jedis.get(deleteKeyOfDevice);
				if (connectMsgStr != null) {
					ConnectMsg connectMsg = StringUtil.convert2Object(connectMsgStr, ConnectMsg.class);
					String socketId = connectMsg.getSocketId();
					jedis.del(deleteKeyOfDevice);
					jedis.srem(deleteKeyOfLink, socketId);
				} else {
					logger.error("error deviceId [{}] in select [{}] key [{}]", deviceId, 15, deleteKeyOfDevice);
				}
			}
			jedis.del(selectKey);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
		}
	}

	public static String getIp(String eth0) {
		String hostAddress = "127.0.0.1";
		try {
			Enumeration<NetworkInterface> interfaces = null;
			interfaces = NetworkInterface.getNetworkInterfaces();
			while (interfaces.hasMoreElements()) {
				NetworkInterface ni = interfaces.nextElement();
				if (eth0.equalsIgnoreCase(ni.getName())) {
					Enumeration<InetAddress> addresss = ni.getInetAddresses();
					while (addresss.hasMoreElements()) {
						InetAddress ip = addresss.nextElement();
						if (ip != null && ip instanceof Inet4Address) {
							if (ip.getHostAddress().equals("127.0.0.1")) {
								continue;
							}
							return hostAddress = ip.getHostAddress();
						}
					}
				}
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return hostAddress;
	}
	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.context = applicationContext;
	}

}
