package com.tidecloud.dataacceptance.service;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.xml.bind.DatatypeConverter;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.util.concurrent.FailureCallback;
import org.springframework.util.concurrent.ListenableFuture;
import org.springframework.util.concurrent.SuccessCallback;

import com.smartsanitation.common.util.StringUtil;
import com.tidecloud.dataacceptance.entity.ConnectMsg;
import com.tidecloud.dataacceptance.entity.Session;
import com.tidecloud.dataacceptance.entity.SessionManager;
import com.tidecloud.dataacceptance.service.impl.YiTongGpsServerHandler;
import com.tidecloud.dataacceptance.util.FileUtils;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.handler.timeout.IdleStateHandler;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

public  class AcceptanceInboundHandlerAdapter extends ChannelInboundHandlerAdapter {

	protected String ip;

	protected Integer port;

	protected String dataPath;

	protected String prefixName;

	private static final Logger logger = LoggerFactory.getLogger(AcceptanceInboundHandlerAdapter.class);
	
	public static String PREFIX_LINK = "s.";
	public static String PREFIX_LINK_BACK = "s.b.";
	public static String PREFIX_DEVICE = "d.";
	public static String MDC_DEVICEID = "deviceId";
	public static final Integer REDIS_INDEX_LINK = 15;

	public static Map<String, String> channelMap = new HashMap<String, String>();
	public static Map<String, Channel> manageChannelMap = new HashMap<>();
	public static Map<String, Channel> channelMapOfChannelKey = new HashMap<>();
	public static Map<String, String> commandCopy = new HashMap<>();

	public static Map<String, Channel> socketyChannelMap = new HashMap<>();
	public static Map<Channel, String> channelDeviceMap = new HashMap<>();

	private final SessionManager sessionManager = SessionManager.getInstance();

	private static ExecutorService singleThreadPool = Executors.newSingleThreadExecutor();

	private static ExecutorService kafkaSendThreadPool = Executors.newSingleThreadExecutor();
	@Autowired
	private JedisPool jedisPool;
	
	@Autowired
	private KafkaTemplate<Integer, byte[]> kafkaTemplate;

	private String topic;

	public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception{	
		MDC.put("strategyName", this.getPrefixName());
	}

	protected void sendMsg2Kafka(byte[]  msg, String deviceId, Channel channel) {
		if (StringUtils.isNotBlank(deviceId)&&msg.length>0) {
			manageChannel(channel, deviceId);
			kafkaSendThreadPool.execute(() -> sendKakfaMsg(msg, deviceId));
		}
		//singleThreadPool.execute(() -> dataStorage(msg));
	}

	protected void manageChannel(Channel channel, String deviceId) {
		MDC.put(MDC_DEVICEID, deviceId);
		String socketkey = channel.id().asLongText();
		Channel channelInMap = socketyChannelMap.get(socketkey);
		String deviceIdInMap = channelDeviceMap.get(channel);
		if (channelInMap != null && deviceIdInMap != null) {
			logger.debug("device [{}] has link [{}]", deviceId, socketkey);
			return;
		}
		socketyChannelMap.put(socketkey, channel);
		channelDeviceMap.put(channel, deviceId);
		String addressStr = ConnectMsg.ipToLong(ip);
		ConnectMsg cMsg = new ConnectMsg(ip, socketkey);
		
		try (Jedis jedis = jedisPool.getResource()){
			jedis.select(REDIS_INDEX_LINK);
			String insertKey = PREFIX_LINK + addressStr;
			String selectKey = PREFIX_DEVICE + deviceId;
			String insertBackupKey = PREFIX_LINK_BACK + addressStr;
			jedis.sadd(insertKey, socketkey);
			jedis.sadd(insertBackupKey, deviceId);
			jedis.set(selectKey, StringUtil.convert2String(cMsg));
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}


    protected String getMsg(byte[] dataByteArray) {
    	String msg = DatatypeConverter.printHexBinary(dataByteArray);
    	return msg;
    }
	protected void sendKakfaMsg(byte[] dataByteArray,  String deviceId) {
		Integer key = new Integer(deviceId.hashCode());
		MDC.put("strategyName", this.getPrefixName());
		MDC.put(MDC_DEVICEID, deviceId);
		
		String msg = getMsg(dataByteArray);
		ListenableFuture<SendResult<Integer, byte[]>> listenableFuture = kafkaTemplate.send(topic, key, dataByteArray);

		// 发送成功回调
		SuccessCallback<SendResult<Integer,  byte[]>> successCallback = new SuccessCallback<SendResult<Integer,  byte[]>>() {
			@Override
			public void onSuccess(SendResult<Integer,  byte[]> result) {
				MDC.put(MDC_DEVICEID, deviceId);
				MDC.put("strategyName", AcceptanceInboundHandlerAdapter.this.getPrefixName());
				// 成功业务逻辑
				logger.info("发送kafka成功.deviceId:{}, msg:{}", deviceId, msg);
			}
		};
		// 发送失败回调
		FailureCallback failureCallback = new FailureCallback() {
			@Override
			public void onFailure(Throwable ex) {
				MDC.put(MDC_DEVICEID, deviceId);
				MDC.put("strategyName", AcceptanceInboundHandlerAdapter.this.getPrefixName());
				// 失败业务逻辑
				logger.error("发送kafka失败.deviceId:{}, msg:{}", deviceId, msg);
				singleThreadPool.execute(() -> FileUtils.dataStorage(dataByteArray,dataPath,prefixName));
			}
		};
		listenableFuture.addCallback(successCallback, failureCallback);

	}
//	public void dataStorage(byte[] deviceByteArr) {
//		
//		File path = new File(dataPath);
//		File[] listFiles = path.listFiles();
//		boolean isTouch = true;
//		if (listFiles != null) {
//			for (File sonFile : listFiles) {
//				long len = sonFile.length();
//				if (len < TEN_M) {
//					// String fileName = dataPath + prefixName + new
//					// SimpleDateFormat("yyMMddHHmmss").format(new Date());
//					// File file = new File(fileName);
//					writeDevice2File(sonFile, deviceByteArr);
//					logger.info("正在写入数据: " + deviceByteArr);
//					isTouch = false;
//				}
//			}
//		}
//		if (isTouch) {
//			String fileName = dataPath +File.separator+ prefixName + new SimpleDateFormat("yyMMddHHmmss").format(new Date());
//			File file = new File(fileName);
//			writeDevice2File(file, deviceByteArr);
//			logger.info("满10M，创建新的文件 正在写入数据：" + deviceByteArr + "timestamp："
//					+ new SimpleDateFormat("yyMMddHHmmss").format(new Date()));
//		}
//	}
//		
//	 public static void writeDevice2File(File file, byte[] deviceByteArr){
//	      FileOutputStream fos = null;
//	      try  {
//	      	fos = new FileOutputStream(file, true);
//	      	Integer length = deviceByteArr.length;
//	          byte[] lengthBytes = NumUtil.int2bytes(length);
//	          byte[] deviceBytes = deviceByteArr;
//	          byte[] dataBytes = ArrayUtils.addAll(lengthBytes, deviceBytes);
//	          
//	          FileChannel fc = fos.getChannel();
//	          
//	          ByteBuffer bbf = ByteBuffer.wrap(dataBytes);  
//	         // bbf.flip();  
//	          fc.write(bbf) ; 
//	          fc.close();  
//	          fos.flush(); 
//	      } catch (IOException e) {
//	    	  logger.error(e.getMessage());
//	      } finally {
//	      	if (fos != null) {  
//	              try {  
//	                  fos.close();  
//	              } catch (IOException e) {  
//	            	  logger.error(e.getMessage());
//	              }  
//	          }  
//	      }
//	 }
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		//cause.printStackTrace();
		logger.error("acceptor server throw exception:"+cause.getCause().getMessage());
		ctx.close();
	}

	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		Session session = Session.buildSession(ctx.channel());
		sessionManager.put(session.getId(), session);
		logger.info("client linking server session : [{}]", StringUtil.convert2String(session));
	}

	@Override
	public void channelInactive(ChannelHandlerContext ctx) throws Exception {
		Channel channel = ctx.channel();
		if (!channel.isActive()) {
			String deviceId = channelDeviceMap.get(channel);
			if (deviceId != null) {
				channelDeviceMap.remove(channel);
				deleteLinkFromRedis(deviceId);
			}
		}
		super.channelInactive(ctx);
		ctx.close();
		final String sessionId = ctx.channel().id().asLongText();
		Session session = sessionManager.findBySessionId(sessionId);
		this.sessionManager.removeBySessionId(sessionId);
		logger.info("client disconnect server session is : [{}]", StringUtil.convert2String(session));
	}

	protected void deleteLinkFromRedis(String deviceId) {
		String deleteKey = PREFIX_DEVICE + deviceId;
		try (Jedis jedis = jedisPool.getResource()) {
			jedis.select(REDIS_INDEX_LINK);
			String connectMsg = jedis.get(deleteKey);
			if (connectMsg != null) {
				ConnectMsg cmsg = StringUtil.convert2Object(connectMsg, ConnectMsg.class);
				String socketId = cmsg.getSocketId();
				socketyChannelMap.remove(socketId);
				String socketQueryKey = PREFIX_LINK + this.ip;
				jedis.srem(socketQueryKey, socketId);
				jedis.del(deleteKey);
				logger.info("delete link [{}] from redis and memory deviceId is [{}]", socketId, deviceId);
			}
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
		}
	}

	@Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (IdleStateEvent.class.isAssignableFrom(evt.getClass())) {
            IdleStateEvent event = (IdleStateEvent) evt;
            if (event.state() == IdleState.READER_IDLE) {
                Session session = this.sessionManager.removeBySessionId(Session.buildId(ctx.channel()));
                logger.error("server breaking connect session : [{}]", StringUtil.convert2String(session));
                ctx.close();
            }
        }
    }
	public static String byteBufferToString(ByteBuffer buffer) {
		CharBuffer charBuffer = null;
		try {
			Charset charset = Charset.forName("UTF-8");
			CharsetDecoder decoder = charset.newDecoder();
			charBuffer = decoder.decode(buffer);
			buffer.flip();
			return charBuffer.toString();
		} catch (Exception ex) {
			int limit = buffer.limit();
			buffer.flip();
			while (buffer.position() < limit) {
				byte b = buffer.get();
				logger.warn("" + (char)b);
			}
			logger.error(ex.getMessage(), ex);
			return null;
		}
	}

	protected Integer getInteger(String str) {
		if (str == null) {
			return null;
		}
		return Integer.valueOf(str);
	}

	protected Double getDouble(String str) {
		if (str == null) {
			return null;
		}
		return Double.valueOf(str);
	}

	static byte[] int2bytes(int num) {
		byte[] b = new byte[4];
		// int mask=0xff;
		for (int i = 0; i < 4; i++) {
			b[3 - i] = (byte) (num >>> (24 - i * 8));
		}
		return b;
	}

	public void startAcceptor() {
		EventLoopGroup bossGroup = new NioEventLoopGroup();
		EventLoopGroup workerGroup = new NioEventLoopGroup();
		try {
			ServerBootstrap b = new ServerBootstrap();
			b.group(bossGroup, workerGroup).channel(NioServerSocketChannel.class)
					.childHandler(new ChannelInitializer<SocketChannel>() {
						@Override
						protected void initChannel(SocketChannel ch) throws Exception {
							ch.pipeline().addLast("idleStateHandler", new IdleStateHandler(10, 0, 0, TimeUnit.MINUTES));
							ch.pipeline().addLast(AcceptanceInboundHandlerAdapter.this);
						}
					}).option(ChannelOption.SO_BACKLOG, 1024).childOption(ChannelOption.SO_KEEPALIVE, true);

			ChannelFuture f = b.bind(this.getPort()).sync();
			logger.info("start accept service for {}, bind address {}:{}", this.getPrefixName(), this.getIp(),
					this.getPort());
			f.channel().closeFuture().sync();
		} catch (Exception e) {
			logger.error(e.getMessage());
		} finally {
			cleanRedisLinkData();
			workerGroup.shutdownGracefully();
			bossGroup.shutdownGracefully();
		}
	}

	protected void cleanRedisLinkData() {

		try (Jedis jedis = jedisPool.getResource()) {
			jedis.select(YiTongGpsServerHandler.REDIS_INDEX_LINK);
			String addressStr = ConnectMsg.ipToLong(this.ip);
			String selectKey = YiTongGpsServerHandler.PREFIX_LINK_BACK + addressStr;
			Set<String> values = jedis.smembers(selectKey);

			for (String deviceId : values) {
				String deleteKeyOfDevice = YiTongGpsServerHandler.PREFIX_DEVICE + deviceId;
				String deleteKeyOfLink = YiTongGpsServerHandler.PREFIX_LINK + addressStr;
				String connectMsgStr = jedis.get(deleteKeyOfDevice);
				if (connectMsgStr != null) {
					ConnectMsg connectMsg = StringUtil.convert2Object(connectMsgStr, ConnectMsg.class);
					String socketId = connectMsg.getSocketId();
					jedis.del(deleteKeyOfDevice);
					jedis.srem(deleteKeyOfLink, socketId);
				} else {
					logger.error("error deviceId [{}] in select [{}] key [{}]", deviceId, 15, deleteKeyOfDevice);
				}
			}
			jedis.del(selectKey);
		} catch (Exception e) {
			logger.error(e.getLocalizedMessage());
		}
	}

	public String getDataPath() {
		return dataPath;
	}

	public void setDataPath(String dataPath) {
		this.dataPath = dataPath;
	}

	public String getTopic() {
		return topic;
	}

	public void setTopic(String topic) {
		this.topic = topic;
	}

	public String getIp() {
		return ip;
	}

	public void setIp(String ip) {
		this.ip = ip;
	}

	public Integer getPort() {
		return port;
	}

	public void setPort(Integer port) {
		this.port = port;
	}

	public String getPrefixName() {
		return prefixName;
	}

	public void setPrefixName(String prefixName) {
		this.prefixName = prefixName;
	}

}
