package com.tidecloud.dataacceptance.web;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.DatatypeConverter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.smartsanitation.common.response.ResultWrapper;
import com.smartsanitation.common.util.StringUtil;
import com.tidecloud.dataacceptance.common.BitOperator;
import com.tidecloud.dataacceptance.common.Constants;
import com.tidecloud.dataacceptance.entity.ConnectMsg;
import com.tidecloud.dataacceptance.service.impl.BSJGpsServerHandler;

import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
/**   
 * @author: chudk 
 * @date:   2017年10月30日 上午9:44:50   
 */
//@RestController("bsjsetting")
public class BSJDeviceController {
    
    private static final Logger logger = LoggerFactory.getLogger(DeviceController.class);
    
    @Autowired
    private JedisPool jedisPool;
    
    @RequestMapping("cuttingOilAndElecticity")
    public ResultWrapper<?> deviceController(@RequestParam String deviceIds, @RequestParam String commandStr){
        Map<String, ConnectMsg> deviceIdSocketIdMap = getDeviceSocketId(deviceIds);
        if (deviceIdSocketIdMap.isEmpty()) {
            return ResultWrapper.ok();
        }
        try {
            Map<String, Channel> channelMap = BSJGpsServerHandler.socketyChannelMap;
            if (channelMap.isEmpty()) {
                return ResultWrapper.ok();
            }
            for (Map.Entry<String, ConnectMsg> deviceIdSocketIdEntry : deviceIdSocketIdMap.entrySet()) {
                ConnectMsg connectMsg = deviceIdSocketIdEntry.getValue();
                String deviceId = deviceIdSocketIdEntry.getKey();
                Channel channel = channelMap.get(connectMsg.getSocketId());
                if (channel != null && channel.isActive()) {
                    logger.info("start write command to device the address [{}]", channel.remoteAddress());
                    sendCommand(channel, commandStr, deviceId);
                    logger.info("end write command to device the address [{}]", channel.remoteAddress());
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        return null;
    }
    private static void sendCommand(Channel channel, String commandStr, String deviceId) throws Exception{
        byte[] startCommand = new byte[]{(byte)0x81, 0x05, 0x00, 0x01};
        byte[] deviceIdBytes = getDeviceBytes(deviceId);
        byte[] serialBytes = new byte[]{0x00, 0x00};
        byte[] commandByte = new byte[]{0x64};
//        byte[] commandBytes = commandStr.getBytes("GBK");
        byte[] countVerifyBytes = BitOperator.concatAll(startCommand, deviceIdBytes, serialBytes, commandByte);
        int verifyCode = BitOperator.getCheckSum4JT808(countVerifyBytes, 0, countVerifyBytes.length);
        
        byte[] noEscapedBytes = BitOperator.concatAll(Arrays.asList(//
                new byte[] { Constants.PKG_DELIMITER }, // 0x7e
                countVerifyBytes, // 消息头+ 消息体
                BitOperator.integerTo1Bytes(verifyCode), // 校验码
                new byte[] { Constants.PKG_DELIMITER }// 0x7e
        ));
        
        ChannelFuture future = channel.writeAndFlush(Unpooled.copiedBuffer(noEscapedBytes)).sync();
        String copyStr = DatatypeConverter.printHexBinary(noEscapedBytes);
        logger.info("send copy message [{}] >>>> client", copyStr);
        if (!future.isSuccess()) {
            logger.error("发送数据出错:{}", future.cause());
        }
    }
    
    private Map<String, ConnectMsg> getDeviceSocketId(String deviceIds) {
        Map<String, ConnectMsg> deviceIdSocketIdMap = new HashMap<String, ConnectMsg>();
        String[] deviceIdsArray = deviceIds.split(",");
        Jedis jedis = jedisPool.getResource();
        try {
            jedis.select(BSJGpsServerHandler.REDIS_INDEX_LINK);
            for (String deviceId : deviceIdsArray) {
                String selectKey = BSJGpsServerHandler.PREFIX_DEVICE + deviceId;
                String result = jedis.get(selectKey);
                if (result != null) {
                    ConnectMsg connectMsg = StringUtil.convert2Object(result, ConnectMsg.class);
                    deviceIdSocketIdMap.put(deviceId, connectMsg);
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        } finally {
            jedisPool.returnResource(jedis);
        }
        return deviceIdSocketIdMap;
    }
    
    private static byte[] getDeviceBytes(String deviceId) {
        if ((deviceId.length() / 2) != 3) {
            deviceId = "0" + deviceId;
        }
        return BitOperator.hexToBytes(deviceId, 6);
    }
    
    private static byte[] getCommandBytes(String deviceId) {
        return BitOperator.hexToBytes(deviceId, 2);
    }
}